"""
Copyright 2021-2023 MICRORISC s.r.o.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from logging import Logger
from Setter.Common.Command import Command
from Setter.Exceptions.ServiceErrorException import ServiceErrorException
from Setter.Logging import Logging


class ServiceManager:
	"""
	Systemd service manager class.

	Methods
	-------
	exists(service: str) -> bool:
		Check if service exists.
	is_running(service: str) -> bool:
		Check if service is running.
	restart(service: str) -> None:
		Restart service.
	start(service: str) -> None:
		Start service.
	"""

	__logger: Logger = Logging.get_logger()

	@classmethod
	def exists(cls, service: str) -> bool:
		"""
		Check if service exists.

		Parameters
		----------
		service: str
			service name

		Returns
		-------
		bool
			true if service exists, false otherwise
		"""
		command = f'systemctl status {service}.service'
		result = Command.execute(command, True)
		return result.returncode != 4

	@classmethod
	def is_running(cls, service: str) -> bool:
		"""
		Check if service is running.

		Parameters
		----------
		service: str
			service name

		Returns
		-------
		bool
			true if service is running, false otherwise
		"""
		command = f'systemctl is-active {service}.service'
		result = Command.execute(command, True)
		return result.returncode == 0

	@classmethod
	def restart(cls, service: str) -> None:
		"""
		Restart service.

		Parameters
		----------
		service: str
			service name

		Returns
		-------
		None

		Exceptions
		----------
		ServiceErrorException
			raised if an error occurs during service restart
		"""
		command = f'systemctl restart {service}.service'
		result = Command.execute(command, True)
		if result.returncode != 0:
			cls.__logger.error(result.stderr)
			raise ServiceErrorException(result.stderr)

	@classmethod
	def start(cls, service: str) -> None:
		"""
		Start service.

		Parameters
		----------
		service: str
			service name

		Returns
		-------
		None

		Exceptions
		----------
		ServiceErrorException
			raised if an error occurs during service start
		"""
		command = f'systemctl start {service}.service'
		result = Command.execute(command, True)
		if result.returncode != 0:
			cls.__logger.error(result.stderr)
			raise ServiceErrorException(result.stderr)
