"""
Copyright 2021-2023 MICRORISC s.r.o.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
import json
import os.path
from logging import Logger

from influxdb import InfluxDBClient

from Setter.Common.PasswordGenerator import PasswordGenerator
from Setter.InfluxDbConfig import InfluxDbConfig
from Setter.Logging import Logging
from Setter.MosquittoConfig import MosquittoConfig


class InfluxDbBridgeConfig:
	"""
	IQRF MQTT-InfluxDB bridge config.

	Methods
	-------
	configure() -> None:
		Configure IQRF MQTT-InfluxDB bridge.
	configure_influx() -> None:
		Configure InfluxDB database and user.
	"""

	__config_file = '/etc/iqrf-gateway-influxdb-bridge/config.json'
	__logger: Logger = Logging.get_logger()
	__influx_db = 'iqrf'
	__influx_username = 'iqrf-gateway-influxdb-bridge'
	__influx_password = ''
	__mqtt_username = 'iqrf-gateway-influxdb-bridge'
	__mqtt_password = ''

	@classmethod
	def configure(cls) -> None:
		"""
		Configure IQRF MQTT-InfluxDB bridge.

		Parameters
		----------
		None

		Returns
		-------
		None
		"""
		cls.__logger.info('Configuring IQRF MQTT-InfluxDB bridge.')
		if not os.path.exists(cls.__config_file):
			cls.__logger.warning('IQRF MQTT-InfluxDB bridge configuration file not found.')
			return
		cls.__influx_password = PasswordGenerator.generate()
		cls.__mqtt_password = PasswordGenerator.generate()
		cls.configure_influx()
		cls.configure_mqtt()
		with open(cls.__config_file, 'r+', encoding='UTF-8') as file:
			config = json.load(file)
			config['influx']['user'] = cls.__influx_username
			config['influx']['password'] = cls.__influx_password
			config['influx']['bucket'] = cls.__influx_db
			config['mqtt']['host'] = 'localhost'
			config['mqtt']['port'] = 1883
			config['mqtt']['client'] = 'mqtt-influxdb-bridge'
			config['mqtt']['topics'] = ['gateway/+/iqrf/responses']
			config['mqtt']['user'] = cls.__mqtt_username
			config['mqtt']['password'] = cls.__mqtt_password
			file.seek(0)
			json.dump(config, file, indent='\t', separators=(',', ': '), sort_keys=True)
			file.truncate()
			file.close()

	@classmethod
	def configure_influx(cls) -> None:
		"""
		Configure InfluxDB database and user.

		Parameters
		----------
		None

		Returns
		-------
		None
		"""
		client: InfluxDBClient = InfluxDbConfig.create_client()
		client.create_database(cls.__influx_db)
		if not any(user.get('user') == cls.__influx_username for user in client.get_list_users()):
			client.create_user(cls.__influx_username, cls.__influx_password)
		else:
			client.set_user_password(cls.__influx_username, cls.__influx_password)
		client.grant_privilege('all', cls.__influx_db, cls.__influx_username)
		client.close()

	@classmethod
	def configure_mqtt(cls) -> None:
		"""
		Configure MQTT user.

		Parameters
		----------
		None

		Returns
		-------
		None

		"""
		MosquittoConfig.create_user(cls.__mqtt_username, cls.__mqtt_password)
