"""
Copyright 2021-2023 MICRORISC s.r.o.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
import logging
import logging.handlers
from logging import Logger, _nameToLevel as LogNameToLevel
from json.decoder import JSONDecodeError
from systemd.journal import JournalHandler


class Logging:
	"""
	Logging configuration, logger fetch and error formatting class.

	Methods
	-------
	get_logger() -> Logger:
		Initialize and return a new logger object.
	io_error(error: IOError) -> str:
		Create and return IO error log string.
	json_error(error: JSONDecodeError, file: str) -> str:
		Create and return JSON file validation error log string.
	daemon_api_error(error: str) -> str
		Create and return Daemon API response error string.
	"""

	__log_path = '/var/log/iqrf-gateway-setter.log'
	__file_format = '[%(asctime)s]: [%(name)s.%(filename)s:%(lineno)s]: [%(levelname)s] %(message)s'
	__journal_format = '[%(levelname)s] %(message)s'
	__log_level = logging.INFO

	@classmethod
	def get_logger(cls) -> Logger:
		"""
		Initialize and return a new logger object.

		Parameters
		----------
		None

		Returns
		-------
		Logger
			Logger object.
		"""
		logger = logging.getLogger(__name__)
		if not logger.hasHandlers():
			logger.propagate = False
			file_handler = logging.handlers.RotatingFileHandler(cls.__log_path, maxBytes=4000000, backupCount=5, mode='a', delay=True)
			file_handler.setFormatter(logging.Formatter(cls.__file_format))
			journal_handler = JournalHandler()
			journal_handler.setFormatter(logging.Formatter(cls.__journal_format))
			logger.addHandler(file_handler)
			logger.addHandler(journal_handler)
			logger.setLevel(cls.__log_level)
		return logger

	@classmethod
	def set_level(cls, level: str) -> None:
		"""
		Set logging level.

		Parameters
		----------
		level: str
			Logging level.

		Returns
		-------
		None
		"""
		cls.__log_level = LogNameToLevel[level]
		logger = logging.getLogger()
		for handler in logger.handlers:
			if isinstance(handler, (type(JournalHandler()), type(logging.StreamHandler()))):
				handler.setLevel(cls.__log_level)
		logger.info('Log level is se to %s', level)

	@staticmethod
	def io_error(error: IOError) -> str:
		"""
		Create and return IO error log string.

		Parameters
		----------
		error: IOError
			io exception object

		Returns
		-------
		str
			IO error log string.
		"""
		return f'{error.strerror}: {error.filename}.'

	@staticmethod
	def json_error(error: JSONDecodeError, file: str) -> str:
		"""
		Create and return JSON file validation error log string.

		Parameters
		----------
		error: JSONDecodeError
			json decoding error
		file: str
			path to validated file

		Returns
		-------
		str
			JSON file validation error log string.
		"""
		return f'{error.msg} at line {error.lineno} column {error.colno} (char {error.pos}) in file {file}.'

	@staticmethod
	def daemon_api_error(error: str) -> str:
		"""
		Create and return Daemon API response error string.

		Parameters
		----------
		error: str
			error string

		Returns
		-------
		str
			Daemon API response error string.
		"""
		return f'Daemon API request failed: {error}.'
