"""
Copyright 2021-2023 MICRORISC s.r.o.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
import json
import os
from logging import Logger

import bcrypt

from Setter.Common.PasswordGenerator import PasswordGenerator
from Setter.Exceptions.CommandException import CommandException
from Setter.Exceptions.IOErrorException import IOErrorException
from Setter.Logging import Logging
from Setter.MosquittoConfig import MosquittoConfig
from Setter.WebappConfig import WebappConfig


class NodeRedConfig:
	"""
	Node-RED configuration class.

	Methods
	-------
	configure() -> None:
		Configure Node-RED, create webapp API key.
	configure_access(password: str) -> None:
		Configure access password.
	"""

	__token_file = '/etc/iqrf-gateway-setter/node-red-token'
	__settings_file = '/root/.node-red/settings.js'
	__logger: Logger = Logging.get_logger()
	__mqtt_username = 'node-red'
	__mqtt_password = ''

	@classmethod
	def configure(cls) -> None:
		"""
		Configure Node-RED, create webapp API key.

		Generated API key is stored in /etc/iqrf-gateway-setter/node-red-token.

		Parameters
		----------
		None

		Returns
		-------
		None

		Exceptions
		----------
		CommandException
			raised if command to generate webapp API key fails
		IOErrorException
			raised if API key cannot be saved to file
		"""
		cls.__logger.info('Configuring Node-RED.')
		cls.configure_webapp_api()
		cls.configure_mqtt()

	@classmethod
	def configure_access(cls, password: str) -> None:
		"""
		Configure access password.

		Parameters
		----------
		password: str
			Password

		Returns
		-------
		None
		"""
		try:
			with open(cls.__settings_file, mode='r', encoding='utf-8') as file:
				settings = file.read()
			password_hash = bcrypt.hashpw(password.encode(), bcrypt.gensalt(8)).decode()
			settings = settings.replace('$2b$08$GL/ytfjH2scn7AbDwQN/D.IbMEiGbQg5PbgSCCrByyWcEerLC3RYW', password_hash, 1)

			with open(cls.__settings_file, mode='w', encoding='utf-8') as file:
				file.write(settings)
		except IOError as io_error:
			error_str = Logging.io_error(io_error)
			cls.__logger.error(error_str)
			raise IOErrorException(error_str) from io_error

	@classmethod
	def configure_mqtt(cls) -> None:
		"""
		Configure MQTT connection.

		Parameters
		----------
		None

		Returns
		-------
		None

		"""
		cls.__logger.info('Configuring MQTT connection for Node-RED.')
		cls.__mqtt_password = PasswordGenerator.generate()
		MosquittoConfig.create_user(cls.__mqtt_username, cls.__mqtt_password)
		with open('/root/.node-red/broker.json', 'r+', encoding='UTF-8') as file:
			config = json.load(file)
			config['host'] = 'localhost'
			config['port'] = 1883
			config['username'] = cls.__mqtt_username
			config['password'] = cls.__mqtt_password
			config['keepalive'] = 60
			file.seek(0)
			json.dump(config, file, indent='\t', separators=(',', ': '), sort_keys=True)
			file.truncate()
			file.close()

	@classmethod
	def configure_webapp_api(cls) -> None:
		"""
		Create and configure webapp API key.

		Generated API key is stored in /etc/iqrf-gateway-setter/node-red-token.

		Parameters
		----------
		None

		Returns
		-------
		None

		Exceptions
		----------
		CommandException
			raised if command to generate webapp API key fails
		IOErrorException
			raised if API key cannot be saved to file
		"""
		cls.__logger.info('Creating IQRF Gateway Webapp API key for Node-RED.')
		result = WebappConfig.run_manager_command('api-key:add -n --no-formatting -d nodered')
		if result.returncode != 0:
			error_str = f'Failed to generate API key for Node-RED: {result.stderr}.'
			cls.__logger.error(error_str)
			raise CommandException(error_str)
		api_key = result.stdout.strip()

		try:
			with open(cls.__token_file, mode='w', encoding='utf-8') as file:
				file.write(api_key)
			old_path = '/etc/iqrf-gateway-webapp/node-red-token'
			if os.path.exists(old_path):
				os.remove(old_path)
			os.symlink(cls.__token_file, old_path)
		except IOError as io_error:
			error_str = Logging.io_error(io_error)
			cls.__logger.error(error_str)
			raise IOErrorException(error_str) from io_error
